// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.workspaceModel.ide

import com.intellij.openapi.components.service
import com.intellij.openapi.project.Project
import com.intellij.openapi.util.registry.Registry
import com.intellij.workspaceModel.storage.VersionedEntityStorage
import com.intellij.workspaceModel.storage.WorkspaceEntityStorageBuilder

/**
 * Provides access to the storage which holds workspace model entities.
 */
interface WorkspaceModel {
  val entityStorage: VersionedEntityStorage

  /**
   * Modifies the current model by calling [updater] and applying it to the storage. Requires write action.
   */
  fun <R> updateProjectModel(updater: (WorkspaceEntityStorageBuilder) -> R): R

  /**
   * Update project model without the notification to message bus and without resetting accumulated changes.
   *
   * This method doesn't require write action.
   */
  fun <R> updateProjectModelSilent(updater: (WorkspaceEntityStorageBuilder) -> R): R

  companion object {
    @JvmStatic
    val isEnabled: Boolean
      get() = Registry.`is`("ide.new.project.model")

    @JvmStatic
    fun getInstance(project: Project): WorkspaceModel = project.service()
  }
}
